local L = AceLibrary("AceLocale-2.2"):new("Capping")

function Capping:StartAV()
	if not self.avdata then
		self.avdata = {
			["graveyard"]	= "Interface\\Icons\\Spell_Shadow_AnimateDead",
			["tower"] 	= "Interface\\Icons\\Ability_Hunter_Quickshot",
			["tree"]		= "Interface\\Icons\\Spell_Nature_NaturesBlessing",
			["ice"]		= "Interface\\Icons\\Spell_Frost_Glacier",
		}
	end
	
	self:RegisterEvent("CHAT_MSG_MONSTER_YELL", "AVAssaults")
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL", "CheckStartTimer")
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE", "SnowfallAlliance") 
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE", "SnowfallHorde") 

	self:RegisterEvent("GOSSIP_SHOW", "AVQuests")
	self:RegisterEvent("QUEST_PROGRESS", "AVQuests")
	self:RegisterEvent("QUEST_COMPLETE", "AVQuests")
	
	-- experimental syncing, will remove if I don't get feedback if it works
	self:RegisterComm(self.commPrefix, "BATTLEGROUND", "SyncBar")
end

function Capping:SyncBar(prefix, sender, distribution, name, maxTime, elapsed, color)
	if name and not self:CandyBarStatus(name) then
		local icon
		if name == L["Ivus begins moving"] then icon = self.avdata.tree
		elseif name == L["Lokholar begins moving"] then icon = self.avdata.ice
		else icon = self.avdata[self:NodeType(name)] 
		end
		self:StartBar(name, maxTime or 303, icon, color or "lightgrey")
		self:SetCandyBarTimeLeft(name, maxTime - elapsed)
	end
end

function Capping:SyncAV()
	for value,color in pairs(self.activebars) do
		local active, totalTime, elapsed, running = self:CandyBarStatus(value)
		if active and running then
			self:SendCommMessage("BATTLEGROUND", value, totalTime, elapsed, color)
		end
	end
end

function Capping:AVAssaults()
	if string.find(arg1, L["avunderattack"]) then
		local node, faction
		_,_,node = string.find(arg1, L["avunderattack"])
		if string.find(arg1, L["Horde"]) then faction = L["Horde"] else faction = L["Alliance"] end
		node = string.gsub(node, L["The "], "")
		if faction == L["Horde"] then 
			self:StopBar(node)
			self:StartBar(node, 303, self.avdata[self:NodeType(node)], "Red")
		else
			self:StopBar(node)
			self:StartBar(node, 303, self.avdata[self:NodeType(node)], "Blue")
		end
	elseif string.find(arg1, L["avtaken"]) then
		local _,_,node = string.find(arg1, L["avtaken"])
		node = string.gsub(node, L["The "], "")
		self:StopBar(node)
	elseif string.find(arg1, L["avdestroyed"]) then
		local _,_,node = string.find(arg1, L["avdestroyed"])
		node = string.gsub(node, L["The "], "")
		self:StopBar(node)
	elseif string.find(arg1, L["Wicked, wicked, mortals!"]) then 
		self:StartBar(L["Ivus begins moving"], 603, self.avdata.tree, "Blue")
	elseif string.find(arg1, L["The Ice Lord has arrived!"]) or string.find(arg1, L["WHO DARES SUMMON LOKHOLAR"]) then
		self:StartBar(L["Lokholar begins moving"], 603, self.avdata.ice, "Red")
	end
end			

function Capping:SnowfallHorde()
	--Initial capture of Snowfall for horde
	if string.find(strlower(arg1), strlower(L["Snowfall Graveyard"])) then
		self:StopBar(L["Snowfall Graveyard"])
		self:StartBar(L["Snowfall Graveyard"], 303, self.avdata.graveyard, "Red")
	end
end

function Capping:SnowfallAlliance()
	--Initial capture of Snowfall for alliance
	if string.find(strlower(arg1), strlower(L["Snowfall Graveyard"])) then
		self:StopBar(L["Snowfall Graveyard"])
		self:StartBar(L["Snowfall Graveyard"], 303, self.avdata.graveyard, "Blue")
	end
end

-- determines if a node's a graveyard or tower
function Capping:NodeType(node)
	-- check for towers, if not...assume it's a graveyard
	if string.find(strlower(node), strlower(L["Bunker"])) or string.find(strlower(node), strlower(L["Tower"])) then
		return "tower"
	end
	return "graveyard"
end

function Capping:AVQuests()
	if not self.db.profile.avquest then return end
	local target = UnitName("target")
	if event == "GOSSIP_SHOW" or event=="QUEST_PROGRESS" then
		if target == L["Smith Regzar"] or target == L["Murgot Deepforge"] then
			-- Open Quest to Smith or Murgot
			if GetGossipOptions() and string.find(GetGossipOptions(), L["Upgrade to"] ) then
				SelectGossipOption(1)
			elseif GetItemCount(17422) > 20 then -- Armor Scraps 17422
				SelectGossipAvailableQuest(1)
			end
		elseif target == L["Primalist Thurloga"] then
			local num = GetItemCount(17306) -- Stormpike Soldier's Blood 17306
			if num >= 5 then
				SelectGossipAvailableQuest(2)
			elseif num > 0 then
				SelectGossipAvailableQuest(1)
			end
		elseif target == L["Arch Druid Renferal"] then
			local num = GetItemCount(17423) -- Storm Crystal 17423
			if num >= 5 then
				SelectGossipAvailableQuest(2)
			elseif num > 0 then
				SelectGossipAvailableQuest(1)
			end
		elseif target == L["Stormpike Ram Rider Commander"] then
			if GetItemCount(17643) > 0 then -- Frost Wolf Hide 17643
				SelectGossipAvailableQuest(1)
			end
		elseif target == L["Frostwolf Wolf Rider Commander"] then
			if GetItemCount(17642) > 0 then -- Alterac Ram Hide 17642
				SelectGossipAvailableQuest(1)
			end
		end
	end
	if event == "QUEST_PROGRESS" then
		if (target == L["Wing Commander Jeztor"] and GetItemCount(17327) == 0) or -- Stormpike Lieutenant's Flesh 17327
		   (target == L["Wing Commander Guse"] and GetItemCount(17326) == 0) or -- Stormpike Soldier's Flesh 17326
		   (target == L["Wing Commander Mulverick"] and GetItemCount(17328) == 0) or -- Stormpike Commander's Flesh 17328
		   (target == L["Wing Commander Vipore"] and GetItemCount(17503) == 0) or -- Frostwolf Lieutenant's Medal 17503
		   (target == L["Wing Commander Slidore"] and GetItemCount(17502) == 0) or -- Frostwolf Soldier's Medal 17502
		   (target == L["Wing Commander Ichman"] and GetItemCount(17504) == 0) then -- Frostwolf Commander's Medal 17504
			return
	   	end
	   	CompleteQuest()
	elseif event == "QUEST_COMPLETE" then
		GetQuestReward(0)
	end
end

