local L = AceLibrary("AceLocale-2.2"):new("Capping")
local BC = AceLibrary("Babble-Class-2.2")

Capping = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1", "AceConsole-2.0", "AceDB-2.0", "CandyBar-2.0", "AceComm-2.0")
Capping.commPrefix = "Capping"

function Capping:OnInitialize()
	self:SecureHook("WorldStateScoreFrame_Update")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	
	self:RegisterDB("CappingDB")
	self:RegisterDefaults("profile", {
		av = true,
		avquest = true,
		ab = true,
		wsg = true,
		arena = true,
		port = true,
		wait = false,
		autoshow = true,
		mapscale = 1.3,
		narrow = true,
		texture = "BantoBarReverse",
		fontsize = 10,
		width = 200,
		height = 16,
		scale = 1,
		disableworld = true,
		classcolor = true,
		mainup = false,
		reverse = false,
	} )
	
	self:RegisterChatCommand({ "/capping"}, {
		type = "execute", name = "Capping", desc = "Capping",
		func = function() self:ShowOptions() end,
	})
	
	self:SetCommPrefix(self.commPrefix)
	
	self.currentq = { }
	self.activebars = { }
	self.allbars = { }
	
	self:InitLibraries()
	self:LoadOptions()
	self:CreateBackbone()
	
	if self.db.profile.sbx then
		WorldStateAlwaysUpFrame:ClearAllPoints()
		WorldStateAlwaysUpFrame:SetPoint("TOP", UIParent, "TOPLEFT", self.db.profile.sbx, self.db.profile.sby)
	end
	
	if self.db.profile.disableworld then
		self:Hook("WorldStateFrame_ToggleMinimap", true)
	end
	
	-- S&D support
	if SupplyAndDemand and IsAddOnLoadOnDemand("Capping") then
		self:PLAYER_ENTERING_WORLD()
		self:CheckQueueTimer()
	end
end

function Capping:InitLibraries()
	self.dewdrop = AceLibrary("Dewdrop-2.0")

	self.surface = AceLibrary("Surface-1.0")
	self.surface:Register("BantoBarReverse", "Interface\\AddOns\\Capping\\BantoBarReverse")
	-- had to rename steel cause it wasn't registering for some reason
	self.surface:Register("NotSteel", "Interface\\AddOns\\Capping\\NotSteel")
	self.surface:Register("flatsmooth", "Interface\\AddOns\\Capping\\flatsmooth")
	
	self.paint = AceLibrary("PaintChips-2.0")
	self.paint:RegisterColor("grey", 0.3, 0.3, 0.3)
	self.paint:RegisterColor("lightgrey", 0.6, 0.6, 0.6)
	
end

function Capping:ShowOptions()
	self.dewdrop:Open( CappingFrame,
		'children', function() self.dewdrop:FeedAceOptionsTable(self.opts) end,
		'cursorX', true, 'cursorY', true )
end

function Capping:PLAYER_ENTERING_WORLD()
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("UPDATE_BATTLEFIELD_STATUS", "CheckQueueTimer")
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	self:ScheduleEvent(self.ZoneCheck, 1, self)
end

-- zone change
function Capping:ZONE_CHANGED_NEW_AREA()
	self:ResetAll()
	self:ZoneCheck()
end

-- reset all timers and unregister events
function Capping:ResetAll()
	self:UnregisterAllEvents()
	self:CancelAllScheduledEvents()
	for value,_ in pairs(self.activebars) do
		self:StopBar(value)
		self.activebars[value] = nil
	end
	for value in pairs(self.allbars) do
		if not self:IsCandyBarRegistered(value) then self.allbars[value] = nil end
	end
	self:UnregisterAllComms()
	if TargetAlliance then TargetAlliance:Hide() TargetHorde:Hide() end
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("UPDATE_BATTLEFIELD_STATUS", "CheckQueueTimer")
end

-- check entry to a battleground zone
function Capping:ZoneCheck()
	local currentZone = GetZoneText()
	if currentZone == L["Warsong Gulch"] and self.db.profile.wsg then
		self:StartWSG()
		self:ShowBGMap()
	elseif currentZone == L["Arathi Basin"] and self.db.profile.ab then
		self:StartAB()
		self:ShowBGMap()
	elseif currentZone == L["Alterac Valley"] and self.db.profile.av then
		self:StartAV()
		self:ShowBGMap()
	elseif strfind(currentZone, L["Arena"]) and self.db.profile.arena then
		self:StartArena()
		-- there is no minimap for arenas
	elseif BattlefieldMinimap then
		BattlefieldMinimap:Hide()
	end
end

-- show the battlefield minimap
function Capping:ShowBGMap()
	if not BattlefieldMinimap then LoadAddOn("Blizzard_BattlefieldMinimap") end
	BattlefieldMinimap:SetScale(self.db.profile.mapscale)
	self:SetNarrowMap()
	if self.db.profile.autoshow then BattlefieldMinimap:Show() end
end

-- prevents the bgmap from showing outside of battlegrounds
function Capping:WorldStateFrame_ToggleMinimap()
	local zone = GetZoneText()
	if not self.db.profile.disableworld 
	   or zone == L["Alterac Valley"] or zone == L["Arathi Basin"] or zone == L["Warsong Gulch"] or zone == L["Eye of the Storm"] then
		self.hooks.WorldStateFrame_ToggleMinimap()
	end
end

-- narrow the default battlefield minimap to remove unnecessary areas taking space
function Capping:SetNarrowMap(disable)
	if self.db.profile.narrow and not self.narrowed and not disable then
		BattlefieldMinimap1:Hide()
		BattlefieldMinimap4:Hide()
		BattlefieldMinimap5:Hide()
		BattlefieldMinimap8:Hide()
		BattlefieldMinimap9:Hide()
		BattlefieldMinimap12:Hide()
		BattlefieldMinimapBackground:SetWidth(256 / 2)
		BattlefieldMinimapBackground:SetPoint("TOPLEFT", -12 + 64, 12)
		BattlefieldMinimapCorner:SetPoint("TOPRIGHT", -2 - 64 + 12, 3 + 1)
		BattlefieldMinimapCorner:SetWidth(32 * 0.75)
		BattlefieldMinimapCorner:SetHeight(32 * 0.75)
		BattlefieldMinimapCloseButton:SetPoint("TOPRIGHT", 2 - 64 + 12 - 1, 7)
		BattlefieldMinimapCloseButton:SetWidth(32 * 0.75)
		BattlefieldMinimapCloseButton:SetHeight(32 * 0.75)
		BattlefieldMinimap:SetPoint("TOPLEFT", BattlefieldMinimapTab, "BOTTOMLEFT", 0 - 64, -5)
		self:Hook("MouseIsOver", function(frame, top, bottom, left, right)
			if frame == BattlefieldMinimap then 
				left = left + 64
				right = right - 64
			end
			return self.hooks.MouseIsOver(frame, top, bottom, left, right)
		end, true)
		self.narrowed = 1
	elseif disable or (not self.db.profile.narrow and self.narrowed) then
		-- setting things back to blizz's default
		BattlefieldMinimap1:Show()
		BattlefieldMinimap4:Show()
		BattlefieldMinimap5:Show()
		BattlefieldMinimap8:Show()
		BattlefieldMinimap9:Show()
		BattlefieldMinimap12:Show()
		BattlefieldMinimapBackground:SetWidth(256)
		BattlefieldMinimapBackground:SetPoint("TOPLEFT", -12, 12)
		BattlefieldMinimapCorner:SetPoint("TOPRIGHT", -2, 3)
		BattlefieldMinimapCorner:SetWidth(32)
		BattlefieldMinimapCorner:SetHeight(32)
		BattlefieldMinimapCloseButton:SetPoint("TOPRIGHT", 2, 7)
		BattlefieldMinimapCloseButton:SetWidth(32)
		BattlefieldMinimapCloseButton:SetHeight(32)
		BattlefieldMinimap:SetPoint("TOPLEFT", BattlefieldMinimapTab, "BOTTOMLEFT", 0, -5)
		self:Unhook("MouseIsOver")
		self.narrowed = nil
	end
end

-- scoreboard class color indicators - @ name
function Capping:WorldStateScoreFrame_Update()
	if not self.db.profile.classcolor then return end
	for i = 1, MAX_WORLDSTATE_SCORE_BUTTONS do
		local index = FauxScrollFrame_GetOffset(WorldStateScoreScrollFrame) + i
		if index > GetNumBattlefieldScores() then break end
		local _,_,_,_,_,_,_,_,class = GetBattlefieldScore(index)
		local nametext = getglobal("WorldStateScoreButton" .. i .. "NameButtonName"):GetText()
		getglobal("WorldStateScoreButton" .. i .. "NameButtonName"):SetText(" |cFF"..BC:GetHexColor(class).."@|r "..nametext)
	end
end

-- the 2 min timer to port to a certain battleground
function Capping:CheckQueueTimer()
	if not self.db.profile.port and not self.db.profile.wait then return end
	for map in pairs(self.currentq) do
		self.currentq[map] = "previous"
	end
	for i=1,MAX_BATTLEFIELD_QUEUES do
		local status, map, _ = GetBattlefieldStatus(i)
		if status == "confirm" and self.db.profile.port then
			local active, _, _, running = self:CandyBarStatus(L["Port: "]..map)
			if not active or not running then
				local maxTime = 120
				if strfind(map, L["Arena"]) then maxTime = 60 end
				self:StopBar(L["Queue: "]..map)
				self:StartBar(L["Port: "]..map, maxTime, "Interface\\Icons\\Ability_TownWatch", "yellow", true, true)
			end
			self.currentq[map] = status
			self:SetCandyBarTimeLeft(L["Port: "]..map, GetBattlefieldPortExpiration(i)/1000)
		elseif status == "queued" and self.db.profile.wait then
			local active, _, _, running = self:CandyBarStatus(L["Queue: "]..map)
			if not active or not running then
				self:StartBar(L["Queue: "]..map, GetBattlefieldEstimatedWaitTime(i)/1000, "Interface\\Icons\\INV_Misc_Note_03", "lightgrey", true, true)
				self:SetCandyBarFade(L["Queue: "]..map, 120)
			end
			self.currentq[map] = status
			self:SetCandyBarTimeLeft(L["Queue: "]..map, math.max(1, (GetBattlefieldEstimatedWaitTime(i)-GetBattlefieldTimeWaited(i))/1000))
		end
	end
	for map in pairs(self.currentq) do
		if self.currentq[map] == "previous" then
			self:StopBar(L["Port: "]..map)
			self:StopBar(L["Queue: "]..map)
			self.currentq[map] = nil
		end
	end
	self:UpdateCandyBarGroup("CappingBottom")
	self:UpdateCandyBarGroup("CappingTop")
end

-- timer for when a battleground begins
function Capping:CheckStartTimer()
	if string.find(arg1, L["begins in 1 minute"]) or string.find(arg1, L["begin in 1 minute"]) or string.find(arg1, L["1 minute until"]) or string.find(arg1, L["One minute until"]) then
		self:StartStartTimer(63)
	elseif string.find(arg1, L["begins in 30 seconds"]) or string.find(arg1, L["begin in 30 seconds"]) or string.find(arg1, L["30 seconds until"]) or string.find(arg1, L["Thirty seconds until"]) then
		self:StartStartTimer(63, 32)
	elseif string.find(arg1, L["Fifteen seconds until"]) then
		self:StartStartTimer(63, 16)
	end
end

function Capping:StartStartTimer(total, remain)
	if not self:IsCandyBarRegistered(L["Battleground Begins"]) then
		self:StartBar(L["Battleground Begins"], total, "Interface\\Icons\\Spell_Holy_PrayerOfHealing", "lightgrey")
	end
	if remain then self:SetCandyBarTimeLeft(L["Battleground Begins"], remain) end
end

-- creates the statusbar anchor and registers candybar group
function Capping:CreateBackbone()
	self.mainframe = CreateFrame("Button", "CappingFrame", UIParent)
	self.mainframe:SetHeight(16)
	self.mainframe:SetWidth(200)
	self.mainframe:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	                             tile = true, tileSize = 16,
	                             insets = { left = 0, right = 0, top = 0, bottom = 0 }
	                          })
	self.mainframe:SetBackdropColor(0.3, 0.3, 0.3, 0.7)
	self.mainframe:SetMovable(true)
	self.mainframe:RegisterForDrag("LeftButton")
	self.mainframe:RegisterForClicks("RightButtonUp")
	self.mainframe:SetScript("OnDragStart", function() this:StartMoving() end)
	self.mainframe:SetScript("OnDragStop", function() 
		this:StopMovingOrSizing()
		local a,b,c,d,e = this:GetPoint()
		if a == "TOPLEFT" and c == "TOPLEFT" then
			self.db.profile.x = floor(d + 0.5)
			self.db.profile.y = floor(e + 0.5)
		end
	end)
	self.mainframe:SetScript("OnClick", function()
		if arg1 == "RightButton" then
			Capping:ShowOptions()
		end
	end)
	
	self.mainframe:Hide()
	
	self.mainframe.text = self.mainframe:CreateFontString("CappingFrameText", "OVERLAY")
	self.mainframe.text:SetFont(GameFontHighlightSmall:GetFont())
	self.mainframe.text:SetText("Capping")
	self.mainframe.text:ClearAllPoints()
	self.mainframe.text:SetAllPoints(self.mainframe)
	
	if not self.db.profile.x then
		self.mainframe:SetPoint("CENTER", UIParent, "CENTER", 0, -200)
	else
		self.mainframe:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.db.profile.x, self.db.profile.y)
	end
	
	self:RegisterCandyBarGroup("CappingBottom")
	self:SetCandyBarGroupPoint("CappingBottom", "TOP", self.mainframe, "TOP", 0, -13)
	
	self:RegisterCandyBarGroup("CappingTop")
	self:SetCandyBarGroupPoint("CappingTop", "BOTTOM", self.mainframe, "BOTTOM", 0, 13)
	self:SetCandyBarGroupGrowth("CappingTop", true)
end

-- begin a statusbar timer
-- - name: id of bar and text of bar
-- - duration: full duration of bar
-- - icon: icon location
-- - color: paintchips color
-- - nonactive: set true if you want to continue this bar timer after zoning
-- - separate: set true to anchor to the top instead of bottom
-- - specialText: bar text if different from name
function Capping:StartBar(name, duration, icon, color, nonactive, separate, specialText, reverse)
	if not specialText then
		self:RegisterCandyBar(name, duration, name, icon, color)
	else
		self:RegisterCandyBar(name, duration, specialText, icon, color)
	end
	self:SetCandyBarTexture(name, self.surface:Fetch(self.db.profile.texture))
	self:SetCandyBarWidth(name, self.db.profile.width)
	self:SetCandyBarHeight(name, self.db.profile.height)
	self:SetCandyBarFontSize(name, self.db.profile.fontsize)
	self:SetCandyBarScale(name, self.db.profile.scale)
	self:SetCandyBarFade(name, 1)
	self:SetCandyBarBackgroundColor(name, "Grey", 0.6)
	if self.db.profile.mainup then separate = not separate end
	if self.db.profile.reverse then reverse = not reverse end
	if separate then
		self:RegisterCandyBarWithGroup(name, "CappingTop")
	else
		self:RegisterCandyBarWithGroup(name, "CappingBottom")
	end
	if reverse then self:SetCandyBarReversed(name, true) end
	self:SetCandyBarOnClick(name, function(name, button) self:OnClick(name, button) end)

	self:StartCandyBar(name, true)
	if not nonactive then 
		self.activebars[name] = color
	end
	self.allbars[name] = true
end

-- simply cancels a timer
function Capping:StopBar(name)
	self:StopCandyBar(name)
	self:UnregisterCandyBar(name)
end

-- gives some user friendliness to the Candy Bars
-- - left-click to toggle anchor
-- - shift left-click to send maybe useful info to /bg
-- - ctrl left-click to send to /s
-- - right-click to bring up config menu
-- - shift right-click to cancel timer
function Capping:OnClick(name, button)
	if button == "LeftButton" then
		if IsShiftKeyDown() then
			local _,time,elapsed,_ = self:CandyBarStatus(name)
			SendChatMessage(string.format(self:GetFactionFromBarColor(name)..L["%s - %s remaining"], name, self:FormatChatTime(time-elapsed)), "SAY")
		elseif IsControlKeyDown() then
			local _,time,elapsed,_ = self:CandyBarStatus(name)
			SendChatMessage(string.format(self:GetFactionFromBarColor(name)..L["%s - %s remaining"], name, self:FormatChatTime(time-elapsed)), "BATTLEGROUND")
		else
			if CappingFrame:IsVisible() then CappingFrame:Hide()
			else CappingFrame:Show() end
		end
	elseif button == "RightButton" then
		if IsShiftKeyDown() then self:StopBar(name)
		else Capping:ShowOptions() end
	end
end

function Capping:FormatChatTime(secs)
	return string.format("%d:%02d", secs / 60, math.fmod(secs, 60))
end

function Capping:GetFactionFromBarColor(name)
	local faction = ""
	if self.activebars[name] then
		if strlower(self.activebars[name]) == "red" then faction = L["Horde"]..": "
		elseif strlower(self.activebars[name]) == "blue" then faction = L["Alliance"]..": " end
	end
	return faction
end

