local L = AceLibrary("AceLocale-2.2"):new("Capping")

function Capping:LoadOptions()
	self.opts = {
		type = "group",
		args = {
			header = {
				type = "header",
				name = "Capping",
				order = 1,
			},
			av = {
				type = "group",
				name = L["Alterac Valley"],
				desc = L["Alterac Valley"],
				order = 2,
				args = {
					enable = {
						type = "toggle",
						name = L["Enable"],
						desc = L["Enable"],
						order = 1,
						set = function(v) self.db.profile.av = v self:ToggleMod(v, L["Alterac Valley"]) end,
						get = function() return self.db.profile.av end,
					},
					quests = {
						type = "toggle",
						name = L["Auto quest turnins"],
						desc = L["Enable Alterac Valley automatic quest turnins"],
						order = 2,
						set = function(v) self.db.profile.avquest = v end,
						get = function() return self.db.profile.avquest end,
					},
					sync = {
						type = "execute",
						name = "Sync",
						desc = "Sync",
						order = 3,
						func = function() self:SyncAV() end,
						disabled = function() return not self:IsCommRegistered(self.commPrefix) end,
					},
				},
			},
			ab = {
				type = "group",
				name = L["Arathi Basin"],
				desc = L["Arathi Basin"],
				order = 3,
				args = {
					enable = {
						type = "toggle",
						name = L["Enable"],
						desc = L["Enable"],
						order = 1,
						set = function(v) self.db.profile.ab = v self:ToggleMod(v, L["Arathi Basin"]) end,
						get = function() return self.db.profile.ab end,
					},
				},
			},
			wsg = {
				type = "group",
				name = L["Warsong Gulch"],
				desc = L["Warsong Gulch"],
				order = 4,
				args = {
					enable = {
						type = "toggle",
						name = L["Enable"],
						desc = L["Enable"],
						order = 1,
						set = function(v) self.db.profile.wsg = v self:ToggleMod(v, L["Warsong Gulch"]) end,
						get = function() return self.db.profile.wsg end,
					},
				},
			},
			bars = {
				type = "group",
				name = L["Bar"],
				desc = L["Statusbar options"],
				order = 5,
				args = {
					scale = {
						type = "range",
						name = L["Scale"],
						desc = L["Change statusbar scale"],
						set = function(v) self.db.profile.scale = v self:UpdateActiveBars() end,
						get = function() return self.db.profile.scale end,
						min = 0.5, max = 2, step = 0.1,
						order = 1,
					},
					width = {
						type = "range",
						name = L["Width"],
						desc = L["Change statusbar width"],
						set = function(v) self.db.profile.width = v self:UpdateActiveBars() end,
						get = function() return self.db.profile.width end,
						min = 50, max = 300, step = 10,
						order = 2,
					},
					height = {
						type = "range",
						name = L["Height"],
						desc = L["Change statusbar thickness"],
						set = function(v) self.db.profile.height = v self:UpdateActiveBars() end,
						get = function() return self.db.profile.height end,
						min = 10, max = 40, step = 2,
						order = 3,
					},
					texture = {
						type = "text",
						name = L["Texture"],
						desc = L["Statusbar textures"],
						set = function(v) self.db.profile.texture = v self:UpdateActiveBars() end,
						get = function() return self.db.profile.texture end,
						validate = self.surface:List(),
						order = 4,
					},
					fontsize = {
						type = "range",
						name = L["Font size"],
						desc = L["Change statusbar font size"],
						set = function(v) self.db.profile.fontsize = v self:UpdateActiveBars() end,
						get = function() return self.db.profile.fontsize end,
						min = 8, max = 20, step = 0.2,
						order = 5,
					},
					reverse = {
						type = "toggle",
						name = L["Reverse fill"],
						desc = L["Set statusbars to fill up instead of depleting"],
						set = function(v) self.db.profile.reverse = v self:UpdateActiveBars() end,
						get = function() return self.db.profile.reverse end,
						order = 6,
					},
					flip = {
						type = "toggle",
						name = L["Flip growth"],
						desc = L["Set objective timers to grow up and misc timers to grow down"],
						get = function() return self.db.profile.mainup end,
						set = function(v) self.db.profile.mainup = v end,
						order = 7,
					},
					test = {
						type = "execute",
						name = L["Test"],
						desc = L["Start a test bar"],
						func = function() self:StartBar("Testing", 30, "Interface\\Icons\\Ability_ThunderBolt", "lightgrey") end,
						order = 8,
					},
				},
			},
			bgmap = {
				type = "group",
				name = L["Battlefield Minimap"],
				desc = L["Options for the battlefield minimap"],
				order = 6,
				args = {
					autoshowmap = {
						type = "toggle",
						name = L["Auto show map"],
						desc = L["Auto show the battlefield minimap on entry"],
						set = function(v) self.db.profile.autoshow = v end,
						get = function() return self.db.profile.autoshow end,
						order = 1, 
					},
					narrow = {
						type = "toggle",
						name = L["Narrow map mode"],
						desc = L["Narrow the battlefield minimap, removing some empty space"],
						set = function(v) self.db.profile.narrow = v self:SetNarrowMap() end,
						get = function() return self.db.profile.narrow end,
						order = 2,
					},
					world = {
						type = "toggle",
						name = L["Disable World PVP Map"],
						desc = L["Disable the battlefield minimap from showing outside battlegrounds"],
						set = function(v) self.db.profile.disableworld = v self:ToggleDisableMap() end,
						get = function() return self.db.profile.disableworld end,
						order = 3,
					},
					mapscale = {
						type = "range",
						name = L["Map scale"],
						desc = L["Change the default scale of the battlefield minimap"],
						set = function(v) 
							self.db.profile.mapscale = v 
							if BattlefieldMinimap and BattlefieldMinimap:IsVisible() then 
								BattlefieldMinimap:SetScale(v) 
							end
						end,
						get = function() return self.db.profile.mapscale end,
						min = 0.5, max = 2.0, step = 0.1,
						order = 4,
					},
				},
			},
			other = {
				type = "group",
				name = L["Other"],
				desc = L["Other options"],
				order = 7,
				args = {
					port = {
						type = "toggle",
						name = L["Port Timer"],
						desc = L["Enable timers for port expiration"],
						set = function(v) self.db.profile.port = v end,
						get = function() return self.db.profile.port end,
						order = 1,
					},
					queue = {
						type = "toggle",
						name = L["Wait Timer"],
						desc = L["Enable timers for queue wait time"],
						set = function(v) self.db.profile.wait = v end,
						get = function() return self.db.profile.wait end,
						order = 2,
					},
					scorecolor = {
						type = "toggle",
						name = L["Toggle class color"],
						desc = L["Enable/disable class color indicators on the scoreboard"],
						set = function(v) self.db.profile.classcolor = v   end,
						get = function() return self.db.profile.classcolor end,
						order = 3,
					},
					movescore = {
						type = "execute",
						name = L["Reposition Scoreboard"],
						desc = L["Move the scoreboard to the Capping anchor's CURRENT position"],
						func = function()
							self.db.profile.sbx, self.db.profile.sby = self.db.profile.x, self.db.profile.y
							WorldStateAlwaysUpFrame:ClearAllPoints()
							WorldStateAlwaysUpFrame:SetPoint("TOP", UIParent, "TOPLEFT", self.db.profile.sbx, self.db.profile.sby)
						end,
						order = 4,
					},
				},
			},
			anchor = {
				type = "execute",
				name = L["Show/Hide Anchor"],
				desc = L["Show/Hide the bars anchor (can also left-click a statusbar)"],
				func = function() 
					if CappingFrame:IsVisible() then 
						CappingFrame:Hide()
					else 
						CappingFrame:Show()
					end 
				end,
				order = 8,
			},
		},
	}
	
	self:RegisterEvent("Surface_Registered", function() self.opts.args.bars.args.texture.validate = self.surface:List() end)
end


-- options functions

-- updates active bars with new textures if needed
function Capping:UpdateActiveBars()
	for value in pairs(self.allbars) do
		if self:IsCandyBarRegistered(value) then
			self:SetCandyBarTexture(value, self.surface:Fetch(self.db.profile.texture))
			self:SetCandyBarWidth(value, self.db.profile.width)
			self:SetCandyBarHeight(value, self.db.profile.height)
			self:SetCandyBarFontSize(value, self.db.profile.fontsize)
			self:SetCandyBarScale(value, self.db.profile.scale)
			self:SetCandyBarReversed(value, self.db.profile.reverse)
		else
			self.allbars[value] = nil
		end
	end
	self:UpdateCandyBarGroup("CappingBottom")
	self:UpdateCandyBarGroup("CappingTop")
end

-- in case a user wants to enable/disable the current bg's mod
function Capping:ToggleMod(enable, zone)
	if GetZoneText() == zone then
		if not enable then self:ResetAll()
		else self:ZoneCheck() end
	end
end

function Capping:ToggleDisableMap()
	if self.db.profile.disableworld and not self:IsHooked("WorldStateFrame_ToggleMinimap") then
		self:Hook("WorldStateFrame_ToggleMinimap", true)
	elseif not self.db.profile.disableworld and self:IsHooked("WorldStateFrame_ToggleMinimap") then	
		self:Unhook("WorldStateFrame_ToggleMinimap")
	end
end
